/** ***********************
封装请求的方法get，post

options选项参数：
 raw: false, 若为真，将后端返回的data对象整个作为返回结果，否则只返回data的data字段
 default: undefined, 若该选项有值，则在接口返回任意错误时固定返回该值作为错误结果

注意：若raw和default均未设置，则在接口返回错误时会直接抛出错误，无法获得返回值

后端封装的错误
// 通用错误
// INVAILD_PARAM: 100, // 无效参数
// NOT_FOUND: 101, // 未找到目标
 **************************/

import axios from 'axios'
import qs from 'qs'
import { Message } from 'element-ui'

import store from '@/store'
import router from '@/router'
import { setCookie } from './auth'
import _ from 'lodash'

axios.defaults.timeout = 5000 * 4 // 超时配置
axios.defaults.baseURL = '' // 配置接口地址
// 配置请求地址
const isProd = process.env.NODE_ENV === 'production'
const isHttps = document.location.protocol === 'https:'
const url_protocol = isHttps ? 'https://' : 'http://'
const urls = {

}
// POST传参序列化(添加请求拦截器)
axios.interceptors.request.use(
  (config) => {
    for (const key in config.data) {
      if (config.data[key] === '' || config.data[key] === null || config.data[key] === undefined) {
        config.data = _.omit(config.data, key)
      } else if (typeof (config.data[key]) === 'string') {
        config.data[key] = _.trim(config.data[key])
      }
    }
    for (const key in config.params) {
      if (config.params[key] === '' || config.params[key] === null || config.params[key] === undefined) {
        config.params = _.omit(config.params, key)
      } else if (typeof (config.params[key]) === 'string') {
        config.params[key] = _.trim(config.params[key])
      }
    }
    if (config.method === 'post' && config.headers['Content-Type'] !== 'multipart/form-data') {
      config.data = qs.stringify(config.data, { skipNulls: true })
    }
    return config
  },
  (error) => {
    console.log('错误的传参')
    return Promise.reject(error)
  }
)

// 返回状态判断(添加响应拦截器)
axios.interceptors.response.use(
  (res) => {
    if (!res.data.success) {
      return Promise.resolve(res)
    }
    return res
  },
  (error) => {
    console.log('网络异常')
    return Promise.reject(error)
  }
)

// 弹出错误对话框
async function errDlg(message, isThrow) {
  await Message({
    message: message,
    type: 'error',
    onClose: function() {
      return false
    }
  })
  if (isThrow) throw new Error(message)
}

// 统一处理返回结果
async function fixResult({ status, data }, options = {}) {
  // 处理服务器非正常返回
  if (status !== 200) {
    const errText = status + '错误'
    await errDlg(errText)
    throw new Error(errText)
  }

  // 处理已知错误
  const hasDefault = typeof options.default !== undefined
  switch (data.code) {
    case 0: // 无错
      return options.raw ? data : data.data
    case 100:
      return data
    case 200: // 该功能需要登录
      errDlg(data.msg)
      router.push('/login')
      return false
    case 201:
      errDlg(data.msg)
      router.push('/login')
      return false
    case 202:
      errDlg(data.msg)
      return false
    case 205:
      errDlg(data.msg)
      setCookie('csp-guid', '')
      router.push('/login')
      return false
    case 300:
      errDlg(data.msg)
      setCookie('csp-password', '')
      setCookie('csp-guid', '')
      router.push('/login')
      return false
    default:
      // 所有其他错误
      if (!options.raw && !hasDefault) {
        throw new Error(data.msg + '(' + data.code + ')')
      }
      break
  }

  return options.raw ? data : options.default
}

// 用get从服务器获取数据
async function apiGet(server, url, params, options) {
  if (!urls[server]) await errDlg('无效的服务器别名：' + server, true)

  // 补充参数
  params = params || {}
  const guid = store.state.guid
  if (!params.t) params.t = new Date().getTime() // 为了避免数据直接从缓存中获取必须传t
  if (guid && !params.guid) params.guid = guid
  params.TTP = 'pc'
  try {
    const res = await axios.get(urls[server] + url, { params })
    return fixResult(res, options)
  } catch (e) {
    await errDlg('服务器访问失败', true)
  }
}

// 用post向服务器发送数据
async function apiPost(server, url, params, options, isOSS) {
  if (!urls[server] && !isOSS) await errDlg('无效的服务器别名：' + server, true)
  // 补充参数
  params = params || {}
  const guid = store.state.guid
  if (guid && !params.guid) params.guid = guid
  params.TTP = 'pc'
  try {
    let res
    if (isOSS) {
      res = await axios.post(server + url, params)
    } else {
      res = await axios.post(urls[server] + url, params)
    }

    return fixResult(res, options)
  } catch (e) {
    console.log(e)
    await errDlg('服务器访问失败', true)
  }
}

export { apiGet, apiPost }
